# -*- Mode: Python; tab-width: 4 -*-

# This is an alternative event loop that supports 'schedulable events'.
# You can specify an event callback to take place after <n> seconds.

# Important usage note: The granularity of the time-check is limited
# by the <timeout> argument to 'go()'; if there is little or no
# activity and you specify a 30-second timeout interval, then the
# schedule of events may only be checked at those 30-second intervals.
# In other words, if you need 1-second resolution, you will have to
# poll at 1-second intervals.  This facility is more useful for longer
# timeouts ("if the channel doesn't close in 5 minutes, then forcibly
# close it" would be a typical usage).

import asyncore
import time

socket_map = asyncore.socket_map

class event_loop:

	def __init__ (self):
		self.events = {}

	def go (self, timeout=30.0):
		events = self.events
		while socket_map:
			now = int(time.time())
			for k,v in events.items():
				if now >= k:
					v (self, now)
					del events[k]
			asyncore.poll (timeout)
			
	def schedule (self, delta, callback):
		now = int (time.time())
		self.events[now + delta] = callback

	def unschedule (self, callback, all=1):
		"unschedule a callback"
		for k,v in self.events:
			if v is callback:
				del self.events[k]
				if not all:
					break

class test (asyncore.dispatcher):
	
	def __init__ (self):
		asyncore.dispatcher.__init__ (self)

	def handle_connect (self):
		print 'Connected!'

	def writable (self):
		return not self.connected

	def connect_timeout_callback (self, event_loop, when):
		if not self.connected:
			print 'Timeout on connect'
			self.close()

	def periodic_thing_callback (self, event_loop, when):
		print 'A Periodic Event has Occurred!'
		# re-schedule it.
		event_loop.schedule (15, self.periodic_thing_callback)
		
if __name__ == '__main__':
	import socket
	el = event_loop()
	t = test ()
	t.create_socket (socket.AF_INET, socket.SOCK_STREAM)
	el.schedule (10, t.connect_timeout_callback)
	el.schedule (15, t.periodic_thing_callback)
	t.connect (('207.30.71.133', 80))
	el.go(1.0)



# -*- Mode: Python; tab-width: 4 -*-

import asynchat
import asyncore
import socket
import string

class proxy_server (asyncore.dispatcher):
	
	def __init__ (self, host, port):
		asyncore.dispatcher.__init__ (self)
		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
		self.set_reuse_addr()
		self.there = (host, port)
		here = ('', port + 9000)
		self.bind (here)
		self.listen (5)

	def handle_accept (self):
		proxy_receiver (self, self.accept())

class proxy_sender (asynchat.async_chat):

	def __init__ (self, receiver, address):
		asynchat.async_chat.__init__ (self)
		self.receiver = receiver
		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
		self.buffer = ''
		self.set_terminator ('\n')
		self.connect (address)

	def handle_connect (self):
		print 'Connected'

	def collect_incoming_data (self, data):
		self.buffer = self.buffer + data

	def found_terminator (self):
		data = self.buffer
		self.buffer = ''
		print '==> (%d) %s' % (self.id, repr(data))
		self.receiver.push (data + '\n')

 	def handle_close (self):
 		self.receiver.close()
 		self.close()

class proxy_receiver (asynchat.async_chat):

	channel_counter = 0

	def __init__ (self, server, (conn, addr)):
		asynchat.async_chat.__init__ (self, conn)
		self.set_terminator ('\n')
		self.server = server
		self.id = self.channel_counter
		self.channel_counter = self.channel_counter + 1
		self.sender = proxy_sender (self, server.there)
		self.sender.id = self.id
		self.buffer = ''

	def collect_incoming_data (self, data):
		self.buffer = self.buffer + data
		
	def found_terminator (self):
		data = self.buffer
		self.buffer = ''
		print '<== (%d) %s' % (self.id, repr(data))
		self.sender.push (data + '\n')

 	def handle_close (self):
 		print 'Closing'
 		self.sender.close()
 		self.close()

if __name__ == '__main__':
	import sys
	import string
	if len(sys.argv) < 3:
		print 'Usage: %s <server-host> <server-port>' % sys.argv[0]
	else:
		ps = proxy_server (sys.argv[1], string.atoi (sys.argv[2]))
		asyncore.loop()



# -*- Mode: Python; tab-width: 4 -*-

import asyncore
import socket
import string

class http_client (asyncore.dispatcher):

	def __init__ (self, host, path):
		asyncore.dispatcher.__init__ (self)
		self.path = path
		self.create_socket (socket.AF_INET, socket.SOCK_STREAM)
		self.connect ((host, 80))

	def handle_connect (self):
		self.send ('GET %s HTTP/1.0\r\n\r\n' % self.path)

	def handle_read (self):
		data = self.recv (8192)
		print data

	def handle_write (self):
		pass

if __name__ == '__main__':
	import sys
	import urlparse
	for url in sys.argv[1:]:
		parts = urlparse.urlparse (url)
		if parts[0] != 'http':
			raise ValueError, "HTTP URL's only, please"
		else:
			host = parts[1]
			path = parts[2]
			http_client (host, path)
	asyncore.loop()
